package com.onsiteservice.util;

import cn.jiguang.common.resp.APIConnectionException;
import cn.jiguang.common.resp.APIRequestException;
import cn.jpush.api.JPushClient;
import cn.jpush.api.push.PushResult;
import cn.jpush.api.push.model.Options;
import cn.jpush.api.push.model.Platform;
import cn.jpush.api.push.model.PushPayload;
import cn.jpush.api.push.model.audience.Audience;
import cn.jpush.api.push.model.notification.AndroidNotification;
import cn.jpush.api.push.model.notification.IosAlert;
import cn.jpush.api.push.model.notification.IosNotification;
import cn.jpush.api.push.model.notification.Notification;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

import static org.springframework.http.HttpStatus.OK;

/**
 * @author 潘维吉
 * @date 2020/3/3 17:27
 * @email 406798106@qq.com
 * @description 极光推送工具类
 */
@ConditionalOnProperty(prefix = "jpush", name = {"enabled"}, matchIfMissing = false)
@Service
@Slf4j
public class JPushUtils {
    private static JPushClient jPushClient = null;
    @Value("${spring.profiles.active}")
    private String env;
    @Value("${jpush.app-key}")
    private String appKey;
    @Value("${jpush.master-secret}")
    private String masterSecret;

    public JPushClient getJPushClient() {
        if (jPushClient == null) {
            jPushClient = new JPushClient(masterSecret, appKey);
        }
        return jPushClient;
    }

    public String push(Integer target, Integer pushType, String title, String content, String extras, String[] pushObjectList) {
        PushResult pushResult;
        String errorMsg = null;
        Map<String, String> extrasMap = JSON.parseObject(extras, new TypeReference<HashMap<String, String>>() {
        });
        IosAlert iosAlert = IosAlert.newBuilder().setTitleAndBody(title, null, content).build();
        PushPayload pushPayload = PushPayload.newBuilder()
                .setPlatform(Platform.android_ios())
                .setAudience(pushType == 1 ? Audience.tag(pushObjectList) : Audience.alias(pushObjectList)) // 使用tag或alias
                .setNotification(Notification.newBuilder()
                        .addPlatformNotification(AndroidNotification.newBuilder()
                                .setAlert(content)
                                .setTitle(title)
                                .addExtras(extrasMap)
                                .build()
                        )
                        .addPlatformNotification(IosNotification.newBuilder()
                                .setAlert(iosAlert)
                                .incrBadge(1)
                                .setSound("default")
                                .addExtras(extrasMap)
                                .build()
                        )
                        .build()
                )
                .setOptions(Options.newBuilder()
                        .setApnsProduction("prod".equals(env))
                        .setSendno(1)
                        .build())
                .build();
        try {

            pushResult = this.getJPushClient().sendPush(pushPayload); // 可以根据不同的客户端判断

            if (pushResult.getResponseCode() != OK.value()) {
                errorMsg = pushResult.error.getMessage();
            }
        } catch (APIConnectionException e) {
            errorMsg = "API连接请求失败";
            log.error("推送消息失败: pushPayload={}, exception={}", pushPayload, e);
        } catch (APIRequestException e) {
            errorMsg = e.getErrorMessage();
            log.error("推送消息失败: pushPayload={}, exception={}", pushPayload, e);
        } catch (Exception e) {
            errorMsg = e.getMessage();
            log.error("推送消息失败: pushPayload={}, exception={}", pushPayload, e);
        }
        return errorMsg;
    }
}
